package cn.efunbox.audio.impl;

import cn.efunbox.audio.entity.Album;
import cn.efunbox.audio.entity.BaseOrderEnum;
import cn.efunbox.audio.helper.SortHelper;
import cn.efunbox.audio.page.OnePage;
import cn.efunbox.audio.repository.AlbumRepo;
import cn.efunbox.audio.service.AlbumService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * AlbumServiceImpl
 * Created by xusq on 2018/5/8.
 */
@Service
public class AlbumServiceImpl implements AlbumService {

    @Value("${efunbox.oss.img.url}")
    private String imgURL;

    @Autowired
    private AlbumRepo albumRepo;

    @Override
    public Album GetOne(BigInteger id) {
        return albumRepo.findOne(id);
    }

    @Override
    public OnePage<Album> SearchAll(Album album,Integer page, Integer pageSize) {

        long count = albumRepo.count(album);
        OnePage onePage = new OnePage(count,page,pageSize);

        if (count == 0) {
            return onePage;
        }
        List<Album> albumList = albumRepo.find(album, onePage.getStart(), onePage.getPageSize(), SortHelper.sortMap2Sort(new LinkedHashMap<String, BaseOrderEnum>() {{
            put("modified", BaseOrderEnum.DESC);
        }}));
        albumList.stream().forEach(albumConsumer -> albumConsumer.setImage(imgURL + albumConsumer.getImage()));
        onePage.setList(albumList);
        return onePage;
    }

    @Override
    public Album Insert(Album album) {
        return albumRepo.save(album);
    }

    @Override
    public Album Update(Album album) {
        return albumRepo.update(album);
    }

    @Override
    public List<Album> searchByName(String name) {
        return albumRepo.findByNameLike(name);
    }

    public List<BigInteger> findLikeName(String name) {
        return albumRepo.findIdsByNameLike(name);
    }

}
